@@ -4,10 +4,12 @@ import collections
 import functools
 import itertools
 import multiprocessing
+import multiprocessing.dummy
 import numpy as np
 import operator
 import os
 import signal
+import timeit
 import types
 
 from contextlib import contextmanager
@@ -42,10 +44,11 @@ __all__ = ['all_eq',
            'merge_none',
            'ndarraywrap',
            'one',
-           'openmp_num_threads',
+           'omp_num_threads',
            'operation_assignment',
            'operation_symbol',
            'pi',
+           'pool_threading',
            'product',
            'renumerate',
            'reshape_broadcast',
@@ -58,6 +61,7 @@ __all__ = ['all_eq',
            'strnbytes',
            'strplural',
            'strshape',
+           'Timer',
            'tointtuple',
            'uninterruptible',
            'uninterruptible_if',
@@ -623,7 +627,7 @@ def one(dtype):
     return np.ones((), dtype=dtype)[()]
 
 
-def openmp_num_threads():
+def omp_num_threads():
     n = os.getenv('OMP_NUM_THREADS')
     if n is not None:
         return int(n)
@@ -650,6 +654,34 @@ def pi(dtype):
     return 4 * np.arctan(one(dtype))
 
 
+@contextmanager
+def pool_threading(nthreads=None):
+    if nthreads is None:
+        nthreads = omp_num_threads()
+    try:
+        import mkl
+        old_mkl_num_threads = mkl.get_max_threads()
+        mkl.set_num_threads(1)
+    except ImportError:
+        pass
+    old_omp_num_threads = os.getenv('OMP_NUM_THREADS')
+    os.environ['OMP_NUM_THREADS'] = '1'
+
+    pool = multiprocessing.dummy.Pool(nthreads)
+    yield pool
+
+    pool.close()
+    pool.join()
+    try:
+        mkl.set_num_threads(old_mkl_num_threads)
+    except NameError:
+        pass
+    if old_omp_num_threads is not None:
+        os.environ['OMP_NUM_THREADS'] = old_omp_num_threads
+    else:
+        del os.environ['OMP_NUM_THREADS']
+
+
 def product(a):
     """ Return the product of a arbitrary input, including generators. """
     if isinstance(a, (list, tuple, types.GeneratorType)):
@@ -919,6 +951,81 @@ def strshape(shape, broadcast=None):
     return str(shape).replace(' ', '').replace("'", '')
 
 
+class Timer(object):
+    """
+    Context manager for timing purposes.
+
+    Examples
+    --------
+    >>> import time
+    >>> with Timer('Elapsed time: '):
+    ...     time.sleep(0.1)
+    Elapsed time: 0.100191831589s
+
+    >>> with Timer() as t:
+    ...     time.sleep(0.1)
+    ...     print(t.elapsed)
+    ...     time.sleep(0.1)
+    ... print(t.elapsed)
+    0.100234985352
+    0.200633049011
+
+    >>> t = Timer(cumulative=True)
+    >>> with t:
+    ...     time.sleep(0.1)
+    >>> print(t.elapsed)
+    >>> with t:
+    ...     time.sleep(0.1)
+    >>> print(t.elapsed)
+    0.100238084793
+    0.200490236282
+
+    """
+    def __init__(self, msg=None, cumulative=False, timer=timeit.default_timer):
+        """
+        Parameters
+        ----------
+        cumulative : boolean
+            If True, elapsed times are accumulated.
+        timer : callable
+            A platform specific timer function (time.time for Unix and
+            time.clock for Windows)
+        msg : string
+            If not None, print the elapsed time upon exiting the context.
+
+        """
+        self.cumulative = cumulative
+        self._elapsed = 0.
+        self._level = 0
+        self.timer = timer
+        self.msg = msg
+
+    def __enter__(self):
+        self._level += 1
+        if self._level == 1:
+            if not self.cumulative:
+                self._elapsed = 0.
+            self._start = self.timer()
+        return self
+
+    def __exit__(self, *args):
+        self._level -= 1
+        if self._level > 0:
+            return
+        self._elapsed += self.timer() - self._start
+        if self.msg is not None:
+            print('{}{}s'.format(self.msg, self.elapsed))
+
+    @property
+    def elapsed(self):
+        if self._level == 0:
+            return self._elapsed
+        return self._elapsed + self.timer() - self._start
+
+    def reset(self):
+        self._elapsed = 0
+
+
 def tointtuple(data):
     """Return input as a tuple of int."""
     if data is None:
